@@ -4,7 +4,6 @@
  *
  * Copyright (C) 2001-2004 Novell, Inc.
  * Copyright (C) 2008 Red Hat, Inc.
- * Copyright (C) 2013 Igalia, S.L.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -15,8 +14,6 @@
 #include <string.h>
 
 #include "soup-logger.h"
-#include "soup-connection.h"
-#include "soup-message-private.h"
 #include "soup.h"
 
 /**
@@ -44,9 +41,9 @@
  * > Host: localhost
  * > Content-Type: text/plain
  * > Connection: close
- * >
+ * > 
  * > This is a test.
- *
+ *   
  * &lt; HTTP/1.1 201 Created
  * &lt; Soup-Debug-Timestamp: 1200171744
  * &lt; Soup-Debug: SoupMessage 1 (0x617000)
@@ -55,7 +52,7 @@
  * </screen></informalexample>
  *
  * The <literal>Soup-Debug-Timestamp</literal> line gives the time (as
- * a <type>time_t</type>) when the request was sent, or the response fully
+ * a #time_t) when the request was sent, or the response fully
  * received.
  *
  * The <literal>Soup-Debug</literal> line gives further debugging
@@ -69,25 +66,23 @@
  *
  * Currently, the request half of the message is logged just before
  * the first byte of the request gets written to the network (from the
- * #SoupMessage::starting signal), which means that if you have
+ * #SoupSession::request_started signal), which means that if you have
  * not made the complete request body available at that point, it will
- * not be logged.
- *
- * The response is logged just after the last byte of the response
- * body is read from the network (from the #SoupMessage::got_body or
- * #SoupMessage::got_informational signal), which means that the
- * #SoupMessage::got_headers signal, and anything triggered off it
- * (such as #SoupSession::authenticate) will be emitted
- * <emphasis>before</emphasis> the response headers are actually
- * logged.
- *
- * If the response doesn't happen to trigger the
- * #SoupMessage::got_body nor #SoupMessage::got_informational signals
- * due to, for example, a cancellation before receiving the last byte
- * of the response body, the response will still be logged on the
- * event of the #SoupMessage::finished signal.
+ * not be logged. The response is logged just after the last byte of
+ * the response body is read from the network (from the
+ * #SoupMessage::got_body or #SoupMessage::got_informational signal),
+ * which means that the #SoupMessage::got_headers signal, and anything
+ * triggered off it (such as #SoupSession::authenticate) will be
+ * emitted <emphasis>before</emphasis> the response headers are
+ * actually logged.
  **/
 
+static void soup_logger_session_feature_init (SoupSessionFeatureInterface *feature_interface, gpointer interface_data);
+
+G_DEFINE_TYPE_WITH_CODE (SoupLogger, soup_logger, G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (SOUP_TYPE_SESSION_FEATURE,
+						soup_logger_session_feature_init))
+
 typedef struct {
 	/* We use a mutex so that if requests are being run in
 	 * multiple threads, we don't mix up the output.
@@ -97,7 +92,6 @@ typedef struct {
 	GQuark              tag;
 	GHashTable         *ids;
 
-	SoupSession        *session;
 	SoupLoggerLogLevel  level;
 	int                 max_body_size;
 
@@ -113,28 +107,12 @@ typedef struct {
 	gpointer            printer_data;
 	GDestroyNotify      printer_dnotify;
 } SoupLoggerPrivate;
-
-enum {
-	PROP_0,
-
-	PROP_LEVEL,
-	PROP_MAX_BODY_SIZE,
-
-	LAST_PROP
-};
-
-static SoupSessionFeatureInterface *soup_logger_default_feature_interface;
-static void soup_logger_session_feature_init (SoupSessionFeatureInterface *feature_interface, gpointer interface_data);
-
-G_DEFINE_TYPE_WITH_CODE (SoupLogger, soup_logger, G_TYPE_OBJECT,
-                         G_ADD_PRIVATE (SoupLogger)
-			 G_IMPLEMENT_INTERFACE (SOUP_TYPE_SESSION_FEATURE,
-						soup_logger_session_feature_init))
+#define SOUP_LOGGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SOUP_TYPE_LOGGER, SoupLoggerPrivate))
 
 static void
 soup_logger_init (SoupLogger *logger)
 {
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+	SoupLoggerPrivate *priv = SOUP_LOGGER_GET_PRIVATE (logger);
 
 	g_mutex_init (&priv->lock);
 	priv->tag = g_quark_from_static_string (g_strdup_printf ("SoupLogger-%p", logger));
@@ -144,8 +122,7 @@ soup_logger_init (SoupLogger *logger)
 static void
 soup_logger_finalize (GObject *object)
 {
-	SoupLogger *logger = SOUP_LOGGER (object);
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+	SoupLoggerPrivate *priv = SOUP_LOGGER_GET_PRIVATE (object);
 
 	g_hash_table_destroy (priv->ids);
 
@@ -161,104 +138,14 @@ soup_logger_finalize (GObject *object)
 	G_OBJECT_CLASS (soup_logger_parent_class)->finalize (object);
 }
 
-static void
-soup_logger_set_property (GObject *object, guint prop_id,
-			  const GValue *value, GParamSpec *pspec)
-{
-	SoupLogger *logger = SOUP_LOGGER (object);
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
-
-	switch (prop_id) {
-	case PROP_LEVEL:
-		priv->level = g_value_get_enum (value);
-		break;
-	case PROP_MAX_BODY_SIZE:
-		priv->max_body_size = g_value_get_int (value);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
-static void
-soup_logger_get_property (GObject *object, guint prop_id,
-			   GValue *value, GParamSpec *pspec)
-{
-	SoupLogger *logger = SOUP_LOGGER (object);
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
-
-	switch (prop_id) {
-	case PROP_LEVEL:
-		g_value_set_enum (value, priv->level);
-		break;
-	case PROP_MAX_BODY_SIZE:
-		g_value_set_int (value, priv->max_body_size);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
 static void
 soup_logger_class_init (SoupLoggerClass *logger_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (logger_class);
 
+	g_type_class_add_private (logger_class, sizeof (SoupLoggerPrivate));
+
 	object_class->finalize = soup_logger_finalize;
-	object_class->set_property = soup_logger_set_property;
-	object_class->get_property = soup_logger_get_property;
-
-	/* properties */
-	/**
-	 * SoupLogger:level:
-	 *
-	 * The level of logging output
-	 *
-	 * Since: 2.56
-	 */
-	/**
-	 * SOUP_LOGGER_LEVEL:
-	 *
-	 * Alias for the #SoupLogger:level property, qv.
-	 *
-	 * Since: 2.56
-	 **/
-	g_object_class_install_property (
-		object_class, PROP_LEVEL,
-		g_param_spec_enum (SOUP_LOGGER_LEVEL,
-				    "Level",
-				    "The level of logging output",
-				    SOUP_TYPE_LOGGER_LOG_LEVEL,
-				    SOUP_LOGGER_LOG_MINIMAL,
-				    G_PARAM_READWRITE));
-
-	/**
-	 * SoupLogger:max-body-size:
-	 *
-	 * If #SoupLogger:level is %SOUP_LOGGER_LOG_BODY, this gives
-	 * the maximum number of bytes of the body that will be logged.
-	 * (-1 means "no limit".)
-	 *
-	 * Since: 2.56
-	 */
-	/**
-	 * SOUP_LOGGER_MAX_BODY_SIZE:
-	 *
-	 * Alias for the #SoupLogger:max-body-size property, qv.
-	 *
-	 * Since: 2.56
-	 **/
-	g_object_class_install_property (
-		object_class, PROP_MAX_BODY_SIZE,
-		g_param_spec_int (SOUP_LOGGER_MAX_BODY_SIZE,
-				    "Max Body Size",
-				    "The maximum body size to output",
-				    -1,
-				    G_MAXINT,
-				    -1,
-				    G_PARAM_READWRITE));
 }
 
 /**
@@ -289,12 +176,18 @@ soup_logger_class_init (SoupLoggerClass *logger_class)
  * Returns: a new #SoupLogger
  **/
 SoupLogger *
-soup_logger_new (SoupLoggerLogLevel level, int max_body_size)
+soup_logger_new (SoupLoggerLogLevel level, int max_body_size) 
 {
-	return g_object_new (SOUP_TYPE_LOGGER,
-			     SOUP_LOGGER_LEVEL, level,
-			     SOUP_LOGGER_MAX_BODY_SIZE, max_body_size,
-			     NULL);
+	SoupLogger *logger;
+	SoupLoggerPrivate *priv;
+
+	logger = g_object_new (SOUP_TYPE_LOGGER, NULL);
+
+	priv = SOUP_LOGGER_GET_PRIVATE (logger);
+	priv->level = level;
+	priv->max_body_size = max_body_size;
+
+	return logger;
 }
 
 /**
@@ -333,7 +226,7 @@ soup_logger_set_request_filter (SoupLogger       *logger,
 				gpointer          filter_data,
 				GDestroyNotify    destroy)
 {
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+	SoupLoggerPrivate *priv = SOUP_LOGGER_GET_PRIVATE (logger);
 
 	priv->request_filter         = request_filter;
 	priv->request_filter_data    = filter_data;
@@ -359,7 +252,7 @@ soup_logger_set_response_filter (SoupLogger       *logger,
 				 gpointer          filter_data,
 				 GDestroyNotify    destroy)
 {
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+	SoupLoggerPrivate *priv = SOUP_LOGGER_GET_PRIVATE (logger);
 
 	priv->response_filter         = response_filter;
 	priv->response_filter_data    = filter_data;
@@ -405,7 +298,7 @@ soup_logger_set_printer (SoupLogger        *logger,
 			 gpointer           printer_data,
 			 GDestroyNotify     destroy)
 {
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+	SoupLoggerPrivate *priv = SOUP_LOGGER_GET_PRIVATE (logger);
 
 	priv->printer         = printer;
 	priv->printer_data    = printer_data;
@@ -415,7 +308,7 @@ soup_logger_set_printer (SoupLogger        *logger,
 static guint
 soup_logger_get_id (SoupLogger *logger, gpointer object)
 {
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+	SoupLoggerPrivate *priv = SOUP_LOGGER_GET_PRIVATE (logger);
 
 	return GPOINTER_TO_UINT (g_object_get_qdata (object, priv->tag));
 }
@@ -423,7 +316,7 @@ soup_logger_get_id (SoupLogger *logger, gpointer object)
 static guint
 soup_logger_set_id (SoupLogger *logger, gpointer object)
 {
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+	SoupLoggerPrivate *priv = SOUP_LOGGER_GET_PRIVATE (logger);
 	gpointer klass = G_OBJECT_GET_CLASS (object);
 	gpointer id;
 
@@ -472,14 +365,11 @@ soup_logger_detach (SoupLogger  *logger,
 	soup_session_remove_feature (session, SOUP_SESSION_FEATURE (logger));
 }
 
-static void soup_logger_print (SoupLogger *logger, SoupLoggerLogLevel level,
-			       char direction, const char *format, ...) G_GNUC_PRINTF (4, 5);
-
 static void
 soup_logger_print (SoupLogger *logger, SoupLoggerLogLevel level,
 		   char direction, const char *format, ...)
 {
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+	SoupLoggerPrivate *priv = SOUP_LOGGER_GET_PRIVATE (logger);
 	va_list args;
 	char *data, *line, *end;
 
@@ -535,19 +425,19 @@ soup_logger_print_basic_auth (SoupLogger *logger, const char *value)
 			*p = '*';
 	}
 	soup_logger_print (logger, SOUP_LOGGER_LOG_HEADERS, '>',
-			   "Authorization: Basic [%.*s]", (int)len, decoded);
+			   "Authorization: Basic [%.*s]", len, decoded);
 	g_free (decoded);
 }
 
 static void
 print_request (SoupLogger *logger, SoupMessage *msg,
-	       SoupSocket *socket, gboolean restarted)
+	       SoupSession *session, SoupSocket *socket,
+	       gboolean restarted)
 {
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+	SoupLoggerPrivate *priv = SOUP_LOGGER_GET_PRIVATE (logger);
 	SoupLoggerLogLevel log_level;
 	SoupMessageHeadersIter iter;
 	const char *name, *value;
-	char *socket_dbg;
 	SoupURI *uri;
 
 	if (priv->request_filter) {
@@ -577,43 +467,24 @@ print_request (SoupLogger *logger, SoupMessage *msg,
 	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, '>',
 			   "Soup-Debug-Timestamp: %lu",
 			   (unsigned long)time (0));
-
-	socket_dbg = socket ?
-		g_strdup_printf ("%s %u (%p)",
-				 g_type_name_from_instance ((GTypeInstance *)socket),
-				 soup_logger_get_id (logger, socket), socket)
-		: NULL;
-
 	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, '>',
-			   "Soup-Debug: %s %u (%p), %s %u (%p), %s%s",
-			   g_type_name_from_instance ((GTypeInstance *)priv->session),
-			   soup_logger_get_id (logger, priv->session), priv->session,
+			   "Soup-Debug: %s %u (%p), %s %u (%p), %s %u (%p)%s",
+			   g_type_name_from_instance ((GTypeInstance *)session),
+			   soup_logger_get_id (logger, session), session,
 			   g_type_name_from_instance ((GTypeInstance *)msg),
 			   soup_logger_get_id (logger, msg), msg,
-			   socket_dbg ? socket_dbg : "cached",
+			   g_type_name_from_instance ((GTypeInstance *)socket),
+			   soup_logger_get_id (logger, socket), socket,
 			   restarted ? ", restarted" : "");
-	g_free (socket_dbg);
 
 	if (log_level == SOUP_LOGGER_LOG_MINIMAL)
 		return;
 
 	if (!soup_message_headers_get_one (msg->request_headers, "Host")) {
-		char *uri_host;
-
-		if (strchr (uri->host, ':'))
-			uri_host = g_strdup_printf ("[%s]", uri->host);
-		else if (g_hostname_is_non_ascii (uri->host))
-			uri_host = g_hostname_to_ascii (uri->host);
-		else
-			uri_host = uri->host;
-
 		soup_logger_print (logger, SOUP_LOGGER_LOG_HEADERS, '>',
-				   "Host: %s%c%u", uri_host,
+				   "Host: %s%c%u", uri->host,
 				   soup_uri_uses_default_port (uri) ? '\0' : ':',
 				   uri->port);
-
-		if (uri_host != uri->host)
-			g_free (uri_host);
 	}
 	soup_message_headers_iter_init (&iter, msg->request_headers);
 	while (soup_message_headers_iter_next (&iter, &name, &value)) {
@@ -646,7 +517,7 @@ print_request (SoupLogger *logger, SoupMessage *msg,
 static void
 print_response (SoupLogger *logger, SoupMessage *msg)
 {
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+	SoupLoggerPrivate *priv = SOUP_LOGGER_GET_PRIVATE (logger);
 	SoupLoggerLogLevel log_level;
 	SoupMessageHeadersIter iter;
 	const char *name, *value;
@@ -690,31 +561,16 @@ print_response (SoupLogger *logger, SoupMessage *msg)
 	}
 }
 
-static void
-finished (SoupMessage *msg, gpointer user_data)
-{
-	SoupLogger *logger = user_data;
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
-
-	g_mutex_lock (&priv->lock);
-
-	print_response (logger, msg);
-	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
-
-	g_mutex_unlock (&priv->lock);
-}
-
 static void
 got_informational (SoupMessage *msg, gpointer user_data)
 {
 	SoupLogger *logger = user_data;
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+	SoupLoggerPrivate *priv = SOUP_LOGGER_GET_PRIVATE (logger);
 
 	g_mutex_lock (&priv->lock);
 
-	g_signal_handlers_disconnect_by_func (msg, finished, logger);
 	print_response (logger, msg);
-	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
+	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "");
 
 	if (msg->status_code == SOUP_STATUS_CONTINUE && msg->request_body->data) {
 		SoupLoggerLogLevel log_level;
@@ -733,7 +589,7 @@ got_informational (SoupMessage *msg, gpointer user_data)
 					   "%s", msg->request_body->data);
 		}
 
-		soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
+		soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "");
 	}
 
 	g_mutex_unlock (&priv->lock);
@@ -743,47 +599,16 @@ static void
 got_body (SoupMessage *msg, gpointer user_data)
 {
 	SoupLogger *logger = user_data;
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+	SoupLoggerPrivate *priv = SOUP_LOGGER_GET_PRIVATE (logger);
 
 	g_mutex_lock (&priv->lock);
 
-	g_signal_handlers_disconnect_by_func (msg, finished, logger);
 	print_response (logger, msg);
-	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
+	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "");
 
 	g_mutex_unlock (&priv->lock);
 }
 
-static void
-starting (SoupMessage *msg, gpointer user_data)
-{
-	SoupLogger *logger = SOUP_LOGGER (user_data);
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
-	gboolean restarted;
-	guint msg_id;
-	SoupConnection *conn;
-	SoupSocket *socket;
-
-	msg_id = soup_logger_get_id (logger, msg);
-	if (msg_id)
-		restarted = TRUE;
-	else {
-		soup_logger_set_id (logger, msg);
-		restarted = FALSE;
-	}
-
-	if (!soup_logger_get_id (logger, priv->session))
-		soup_logger_set_id (logger, priv->session);
-
-	conn = soup_message_get_connection (msg);
-	socket = conn ? soup_connection_get_socket (conn) : NULL;
-	if (socket && !soup_logger_get_id (logger, socket))
-		soup_logger_set_id (logger, socket);
-
-	print_request (logger, msg, socket, restarted);
-	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
-}
-
 static void
 soup_logger_request_queued (SoupSessionFeature *logger,
 			    SoupSession *session,
@@ -791,18 +616,44 @@ soup_logger_request_queued (SoupSessionFeature *logger,
 {
 	g_return_if_fail (SOUP_IS_MESSAGE (msg));
 
-	g_signal_connect (msg, "starting",
-			  G_CALLBACK (starting),
-			  logger);
 	g_signal_connect (msg, "got-informational",
 			  G_CALLBACK (got_informational),
 			  logger);
 	g_signal_connect (msg, "got-body",
 			  G_CALLBACK (got_body),
 			  logger);
-	g_signal_connect (msg, "finished",
-			  G_CALLBACK (finished),
-			  logger);
+}
+
+static void
+soup_logger_request_started (SoupSessionFeature *feature,
+			     SoupSession *session,
+			     SoupMessage *msg,
+			     SoupSocket *socket)
+{
+	SoupLogger *logger = SOUP_LOGGER (feature);
+	gboolean restarted;
+	guint msg_id;
+
+	g_return_if_fail (SOUP_IS_SESSION (session));
+	g_return_if_fail (SOUP_IS_MESSAGE (msg));
+	g_return_if_fail (SOUP_IS_SOCKET (socket));
+
+	msg_id = soup_logger_get_id (logger, msg);
+	if (msg_id)
+		restarted = TRUE;
+	else {
+		soup_logger_set_id (logger, msg);
+		restarted = FALSE;
+	}
+
+	if (!soup_logger_get_id (logger, session))
+		soup_logger_set_id (logger, session);
+
+	if (!soup_logger_get_id (logger, socket))
+		soup_logger_set_id (logger, socket);
+
+	print_request (logger, msg, session, socket, restarted);
+	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "");
 }
 
 static void
@@ -812,31 +663,15 @@ soup_logger_request_unqueued (SoupSessionFeature *logger,
 {
 	g_return_if_fail (SOUP_IS_MESSAGE (msg));
 
-	g_signal_handlers_disconnect_by_func (msg, starting, logger);
 	g_signal_handlers_disconnect_by_func (msg, got_informational, logger);
 	g_signal_handlers_disconnect_by_func (msg, got_body, logger);
-	g_signal_handlers_disconnect_by_func (msg, finished, logger);
-}
-
-static void
-soup_logger_feature_attach (SoupSessionFeature *feature,
-			    SoupSession *session)
-{
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (SOUP_LOGGER (feature));
-
-	priv->session = session;
-
-	soup_logger_default_feature_interface->attach (feature, session);
 }
 
 static void
 soup_logger_session_feature_init (SoupSessionFeatureInterface *feature_interface,
 				  gpointer interface_data)
 {
-	soup_logger_default_feature_interface =
-		g_type_default_interface_peek (SOUP_TYPE_SESSION_FEATURE);
-
-	feature_interface->attach = soup_logger_feature_attach;
 	feature_interface->request_queued = soup_logger_request_queued;
+	feature_interface->request_started = soup_logger_request_started;
 	feature_interface->request_unqueued = soup_logger_request_unqueued;
 }
