@@ -105,15 +105,6 @@ soup_session_async_new_with_options (const char *optname1, ...)
 	return session;
 }
 
-static void
-connection_closed (SoupConnection *conn, gpointer session)
-{
-	/* Run the queue in case anyone was waiting for a connection
-	 * to be closed.
-	 */
-	do_idle_run_queue (session);
-}
-
 static void
 message_completed (SoupMessage *msg, gpointer user_data)
 {
@@ -126,77 +117,55 @@ message_completed (SoupMessage *msg, gpointer user_data)
 }
 
 static void
-tunnel_complete (SoupMessageQueueItem *item)
+ssl_tunnel_completed (SoupConnection *conn, guint status, gpointer user_data)
 {
+	SoupMessageQueueItem *tunnel_item = user_data;
+	SoupMessageQueueItem *item = tunnel_item->related;
 	SoupSession *session = item->session;
 
-	soup_message_finished (item->msg);
-	if (item->related->msg->status_code)
-		item->related->state = SOUP_MESSAGE_FINISHING;
-	else
-		soup_message_set_https_status (item->related->msg, item->conn);
+	soup_message_finished (tunnel_item->msg);
+	soup_message_queue_item_unref (tunnel_item);
 
+	if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
+		soup_connection_disconnect (item->conn);
+		soup_message_queue_item_set_connection (item, NULL);
+		soup_message_set_status (item->msg, status);
+	}
+
+	item->state = SOUP_MESSAGE_READY;
 	do_idle_run_queue (session);
-	soup_message_queue_item_unref (item->related);
-	soup_session_unqueue_item (session, item);
 	soup_message_queue_item_unref (item);
 }
 
 static void
-ssl_tunnel_completed (SoupConnection *conn, guint status, gpointer user_data)
+tunnel_message_completed (SoupMessage *tunnel_msg, gpointer user_data)
 {
-	SoupMessageQueueItem *item = user_data;
+	SoupMessageQueueItem *tunnel_item = user_data;
+	SoupSession *session = tunnel_item->session;
+	SoupMessageQueueItem *item = tunnel_item->related;
 
-	if (SOUP_STATUS_IS_SUCCESSFUL (status)) {
-		g_signal_connect (item->conn, "disconnected",
-				  G_CALLBACK (connection_closed), item->session);
-		soup_connection_set_state (item->conn, SOUP_CONNECTION_IDLE);
-		soup_connection_set_state (item->conn, SOUP_CONNECTION_IN_USE);
-
-		item->related->state = SOUP_MESSAGE_READY;
-	} else {
-		if (item->conn)
-			soup_connection_disconnect (item->conn);
-		soup_message_set_status (item->related->msg, SOUP_STATUS_SSL_FAILED);
-	}
-
-	tunnel_complete (item);
-}
-
-static void
-tunnel_message_completed (SoupMessage *msg, gpointer user_data)
-{
-	SoupMessageQueueItem *item = user_data;
-	SoupSession *session = item->session;
-
-	if (item->state == SOUP_MESSAGE_RESTARTING) {
-		soup_message_restarted (msg);
-		if (item->conn) {
-			item->state = SOUP_MESSAGE_RUNNING;
-			soup_session_send_queue_item (session, item, tunnel_message_completed);
+	if (tunnel_item->state == SOUP_MESSAGE_RESTARTING) {
+		soup_message_restarted (tunnel_msg);
+		if (tunnel_item->conn) {
+			tunnel_item->state = SOUP_MESSAGE_RUNNING;
+			soup_session_send_queue_item (session, tunnel_item,
+						      tunnel_message_completed);
 			return;
 		}
 
-		soup_message_set_status (msg, SOUP_STATUS_TRY_AGAIN);
+		soup_message_set_status (tunnel_msg, SOUP_STATUS_TRY_AGAIN);
 	}
 
-	item->state = SOUP_MESSAGE_FINISHED;
+	tunnel_item->state = SOUP_MESSAGE_FINISHED;
+	soup_session_unqueue_item (session, tunnel_item);
 
-	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
-		if (item->conn)
-			soup_connection_disconnect (item->conn);
-		if (msg->status_code == SOUP_STATUS_TRY_AGAIN) {
-			item->related->state = SOUP_MESSAGE_AWAITING_CONNECTION;
-			soup_message_queue_item_set_connection (item->related, NULL);
-		} else
-			soup_message_set_status (item->related->msg, msg->status_code);
-
-		tunnel_complete (item);
-		return;
+	if (SOUP_STATUS_IS_SUCCESSFUL (tunnel_msg->status_code)) {
+		soup_connection_start_ssl_async (item->conn, item->cancellable,
+						 ssl_tunnel_completed, tunnel_item);
+	} else {
+		ssl_tunnel_completed (item->conn, tunnel_msg->status_code,
+				      tunnel_item);
 	}
-
-	soup_connection_start_ssl_async (item->conn, item->cancellable,
-					 ssl_tunnel_completed, item);
 }
 
 static void
@@ -207,43 +176,14 @@ got_connection (SoupConnection *conn, guint status, gpointer user_data)
 
 	if (item->state != SOUP_MESSAGE_CONNECTING) {
 		soup_connection_disconnect (conn);
-		do_idle_run_queue (session);
-		soup_message_queue_item_unref (item);
-		return;
-	}
+	} else if (status != SOUP_STATUS_OK) {
+		soup_session_set_item_status (session, item, status);
+		soup_connection_disconnect (item->conn);
+		soup_message_queue_item_set_connection (item, NULL);
+		item->state = SOUP_MESSAGE_READY;
+	} else
+		item->state = SOUP_MESSAGE_CONNECTED;
 
-	soup_message_set_https_status (item->msg, conn);
-
-	if (status != SOUP_STATUS_OK) {
-		soup_connection_disconnect (conn);
-
-		if (status == SOUP_STATUS_TRY_AGAIN) {
-			soup_message_queue_item_set_connection (item, NULL);
-			item->state = SOUP_MESSAGE_AWAITING_CONNECTION;
-		} else {
-			soup_session_set_item_status (session, item, status);
-			item->state = SOUP_MESSAGE_FINISHING;
-		}
-
-		do_idle_run_queue (session);
-		soup_message_queue_item_unref (item);
-		return;
-	}
-
-	if (soup_connection_is_tunnelled (conn)) {
-		SoupMessageQueueItem *tunnel_item;
-
-		item->state = SOUP_MESSAGE_TUNNELING;
-
-		tunnel_item = soup_session_make_connect_message (session, conn);
-		tunnel_item->related = item;
-		soup_session_send_queue_item (session, tunnel_item, tunnel_message_completed);
-		return;
-	}
-
-	item->state = SOUP_MESSAGE_READY;
-	g_signal_connect (conn, "disconnected",
-			  G_CALLBACK (connection_closed), session);
 	run_queue ((SoupSessionAsync *)session);
 	soup_message_queue_item_unref (item);
 }
@@ -264,10 +204,6 @@ process_queue_item (SoupMessageQueueItem *item,
 
 		switch (item->state) {
 		case SOUP_MESSAGE_STARTING:
-			item->state = SOUP_MESSAGE_AWAITING_CONNECTION;
-			break;
-
-		case SOUP_MESSAGE_AWAITING_CONNECTION:
 			if (!soup_session_get_connection (session, item, should_prune))
 				return;
 
@@ -282,7 +218,34 @@ process_queue_item (SoupMessageQueueItem *item,
 						       got_connection, item);
 			return;
 
+		case SOUP_MESSAGE_CONNECTED:
+			if (soup_connection_is_tunnelled (item->conn)) {
+				SoupMessageQueueItem *tunnel_item;
+
+				soup_message_queue_item_ref (item);
+
+				item->state = SOUP_MESSAGE_TUNNELING;
+
+				tunnel_item = soup_session_make_connect_message (session, item->conn);
+				tunnel_item->related = item;
+				soup_session_send_queue_item (session, tunnel_item, tunnel_message_completed);
+				return;
+			}
+
+			item->state = SOUP_MESSAGE_READY;
+			break;
+
 		case SOUP_MESSAGE_READY:
+			soup_message_set_https_status (item->msg, item->conn);
+			if (item->msg->status_code) {
+				if (item->msg->status_code == SOUP_STATUS_TRY_AGAIN) {
+					soup_message_cleanup_response (item->msg);
+					item->state = SOUP_MESSAGE_STARTING;
+				} else
+					item->state = SOUP_MESSAGE_FINISHING;
+				break;
+			}
+
 			item->state = SOUP_MESSAGE_RUNNING;
 			soup_session_send_queue_item (session, item, message_completed);
 			if (item->new_api)
@@ -308,7 +271,7 @@ process_queue_item (SoupMessageQueueItem *item,
 				item->callback (session, item->msg, item->callback_data);
 			else if (item->new_api)
 				send_request_finished (session, item);
-			do_idle_run_queue (session);
+
 			soup_message_queue_item_unref (item);
 			return;
 
