@@ -1018,6 +1018,8 @@ soup_session_append_queue_item (SoupSession *session, SoupMessage *msg,
 	SoupMessageQueueItem *item;
 	SoupSessionHost *host;
 
+	soup_message_cleanup_response (msg);
+
 	item = soup_message_queue_append (priv->queue, msg, callback, user_data);
 
 	g_mutex_lock (&priv->conn_lock);
@@ -1165,6 +1167,18 @@ connection_disconnected (SoupConnection *conn, gpointer user_data)
 
 	g_mutex_unlock (&priv->conn_lock);
 	g_object_unref (conn);
+
+	SOUP_SESSION_GET_CLASS (session)->kick (session);
+}
+
+static void
+connection_state_changed (GObject *object, GParamSpec *param, gpointer user_data)
+{
+	SoupSession *session = user_data;
+	SoupConnection *conn = SOUP_CONNECTION (object);
+
+	if (soup_connection_get_state (conn) == SOUP_CONNECTION_IDLE)
+		SOUP_SESSION_GET_CLASS (session)->kick (session);
 }
 
 SoupMessageQueueItem *
@@ -1261,6 +1275,9 @@ soup_session_get_connection (SoupSession *session,
 	g_signal_connect (conn, "disconnected",
 			  G_CALLBACK (connection_disconnected),
 			  session);
+	g_signal_connect (conn, "notify::state",
+			  G_CALLBACK (connection_state_changed),
+			  session);
 
 	g_signal_emit (session, signals[CONNECTION_CREATED], 0, conn);
 
@@ -1613,7 +1630,8 @@ soup_session_real_flush_queue (SoupSession *session)
  * soup_session_abort:
  * @session: the session
  *
- * Cancels all pending requests in @session.
+ * Cancels all pending requests in @session and closes all idle
+ * persistent connections.
  **/
 void
 soup_session_abort (SoupSession *session)
